@@ -17,14 +17,14 @@ import ssl
 import sys
 import threading
 import time
-from collections import namedtuple
+from collections import defaultdict, namedtuple
 from struct import pack, unpack
 
 from . import cast_channel_pb2
 from .controllers import BaseController
 from .controllers.media import MediaController
 from .controllers.receiver import ReceiverController
-from .const import CAST_TYPE_CHROMECAST, MESSAGE_TYPE, REQUEST_ID, SESSION_ID
+from .const import MESSAGE_TYPE, REQUEST_ID, SESSION_ID
 from .dial import get_host_from_service
 from .error import (
     ChromecastConnectionError,
@@ -109,7 +109,6 @@ if sys.version_info >= (3, 0):
         """Encodes a python value into JSON format."""
         return json.dumps(data, ensure_ascii=False).encode("utf8")
 
-
 else:
 
     def _json_to_payload(data):
@@ -139,7 +138,7 @@ class ConnectionStatusListener(abc.ABC):
         """Updated connection status."""
 
 
-# pylint: disable=too-many-instance-attributes
+# pylint: disable-next=too-many-instance-attributes
 class SocketClient(threading.Thread):
     """
     Class to interact with a Chromecast through a socket.
@@ -168,13 +167,17 @@ class SocketClient(threading.Thread):
                   pychromecast.start_discovery().
     """
 
-    def __init__(self, host, port=None, cast_type=CAST_TYPE_CHROMECAST, **kwargs):
-        tries = kwargs.pop("tries", None)
-        timeout = kwargs.pop("timeout", None)
-        retry_wait = kwargs.pop("retry_wait", None)
-        services = kwargs.pop("services", None)
-        zconf = kwargs.pop("zconf", None)
-
+    # pylint: disable-next=too-many-arguments
+    def __init__(
+        self,
+        *,
+        cast_type,
+        tries,
+        timeout,
+        retry_wait,
+        services,
+        zconf,
+    ):
         super().__init__()
 
         self.daemon = True
@@ -188,10 +191,11 @@ class SocketClient(threading.Thread):
         self.tries = tries
         self.timeout = timeout or TIMEOUT_TIME
         self.retry_wait = retry_wait or RETRY_TIME
-        self.host = host
-        self.services = services or [None]
+        self.services = services
         self.zconf = zconf
-        self.port = port or 8009
+
+        self.host = "unknown"
+        self.port = 8009
 
         self.source_id = "sender-0"
         self.stop = threading.Event()
@@ -211,7 +215,7 @@ class SocketClient(threading.Thread):
         self.socket = None
 
         # dict mapping namespace on Controller objects
-        self._handlers = {}
+        self._handlers = defaultdict(set)
         self._connection_listeners = []
 
         self.receiver_controller = ReceiverController(cast_type)
@@ -274,8 +278,7 @@ class SocketClient(threading.Thread):
                 retry = retries.get(
                     service, {"delay": self.retry_wait, "next_retry": now}
                 )
-                # If we're connecting to a named service, check if it's time
-                if service and now < retry["next_retry"]:
+                if now < retry["next_retry"]:
                     continue
                 try:
                     self.socket = new_socket()
@@ -286,51 +289,47 @@ class SocketClient(threading.Thread):
                             NetworkAddress(self.host, self.port),
                         )
                     )
-                    # Resolve the service name. If service is None, we're
-                    # connecting directly to a host name or IP-address
-                    if service:
-                        host = None
-                        port = None
-                        host, port, service_info = get_host_from_service(
-                            service, self.zconf
+                    # Resolve the service name.
+                    host = None
+                    port = None
+                    host, port, service_info = get_host_from_service(
+                        service, self.zconf
+                    )
+                    if host and port:
+                        if service_info:
+                            try:
+                                self.fn = service_info.properties[b"fn"].decode("utf-8")
+                            except (AttributeError, KeyError, UnicodeError):
+                                pass
+                        self.logger.debug(
+                            "[%s(%s):%s] Resolved service %s to %s:%s",
+                            self.fn or "",
+                            self.host,
+                            self.port,
+                            service,
+                            host,
+                            port,
                         )
-                        if host and port:
-                            if service_info:
-                                try:
-                                    self.fn = service_info.properties[b"fn"].decode(
-                                        "utf-8"
-                                    )
-                                except (AttributeError, KeyError, UnicodeError):
-                                    pass
-                            self.logger.debug(
-                                "[%s(%s):%s] Resolved service %s to %s:%s",
-                                self.fn or "",
-                                self.host,
-                                self.port,
-                                service,
-                                host,
-                                port,
+                        self.host = host
+                        self.port = port
+                    else:
+                        self.logger.debug(
+                            "[%s(%s):%s] Failed to resolve service %s",
+                            self.fn or "",
+                            self.host,
+                            self.port,
+                            service,
+                        )
+                        self._report_connection_status(
+                            ConnectionStatus(
+                                CONNECTION_STATUS_FAILED_RESOLVE,
+                                NetworkAddress(service, None),
                             )
-                            self.host = host
-                            self.port = port
-                        else:
-                            self.logger.debug(
-                                "[%s(%s):%s] Failed to resolve service %s",
-                                self.fn or "",
-                                self.host,
-                                self.port,
-                                service,
-                            )
-                            self._report_connection_status(
-                                ConnectionStatus(
-                                    CONNECTION_STATUS_FAILED_RESOLVE,
-                                    NetworkAddress(service, None),
-                                )
-                            )
-                            mdns_backoff(service, retry)
-                            # If zeroconf fails to receive the necessary data,
-                            # try next service
-                            continue
+                        )
+                        mdns_backoff(service, retry)
+                        # If zeroconf fails to receive the necessary data,
+                        # try next service
+                        continue
 
                     self.logger.debug(
                         "[%s(%s):%s] Connecting to %s:%s",
@@ -371,7 +370,11 @@ class SocketClient(threading.Thread):
                             self.port,
                         )
                     return
-                except OSError as err:
+
+                # OSError raised if connecting to the socket fails, NotConnected raised
+                # if another thread tries - and fails - to send a message before the
+                # calls to receiver_controller and heartbeat_controller.
+                except (OSError, NotConnected) as err:
                     self.connecting = True
                     if self.stop.is_set():
                         self.logger.error(
@@ -460,10 +463,20 @@ class SocketClient(threading.Thread):
 
     def register_handler(self, handler: BaseController):
         """Register a new namespace handler."""
-        self._handlers[handler.namespace] = handler
+        self._handlers[handler.namespace].add(handler)
 
         handler.registered(self)
 
+    def unregister_handler(self, handler: BaseController):
+        """Register a new namespace handler."""
+        if (
+            handler.namespace in self._handlers
+            and handler in self._handlers[handler.namespace]
+        ):
+            self._handlers[handler.namespace].remove(handler)
+
+        handler.unregistered()
+
     def new_cast_status(self, cast_status):
         """Called when a new cast status has been received."""
         new_channel = self.destination_id != cast_status.transport_id
@@ -481,7 +494,8 @@ class SocketClient(threading.Thread):
             for namespace in self.app_namespaces:
                 if namespace in self._handlers:
                     self._ensure_channel_connected(self.destination_id)
-                    self._handlers[namespace].channel_connected()
+                    for handler in self._handlers[namespace]:
+                        handler.channel_connected()
 
     def _gen_request_id(self):
         """Generates a unique request id."""
@@ -518,20 +532,19 @@ class SocketClient(threading.Thread):
             return
 
         self.heartbeat_controller.reset()
-        self._force_recon = False
         self.logger.debug("Thread started...")
-        try:
-            while not self.stop.is_set():
+        while not self.stop.is_set():
+            try:
                 if self.run_once(timeout=POLL_TIME_BLOCKING) == 1:
                     break
-        except Exception:  # pylint: disable=broad-except
-            self.logger.exception(
-                ("[%s(%s):%s] Unhandled exception in worker thread"),
-                self.fn or "",
-                self.host,
-                self.port,
-            )
-            raise
+            except Exception:  # pylint: disable=broad-except
+                self._force_recon = True
+                self.logger.exception(
+                    "[%s(%s):%s] Unhandled exception in worker thread, attempting reconnect",
+                    self.fn or "",
+                    self.host,
+                    self.port,
+                )
 
         self.logger.debug("Thread done...")
         # Clean up
@@ -696,32 +709,31 @@ class SocketClient(threading.Thread):
                 )
 
             # message handlers
-            try:
-                handled = self._handlers[message.namespace].receive_message(
-                    message, data
-                )
+            for handler in self._handlers[message.namespace]:
+                try:
+                    handled = handler.receive_message(message, data)
 
-                if not handled:
-                    if data.get(REQUEST_ID) not in self._request_callbacks:
-                        self.logger.debug(
-                            "[%s(%s):%s] Message unhandled: %s",
-                            self.fn or "",
-                            self.host,
-                            self.port,
-                            _message_to_string(message, data),
-                        )
-            except Exception:  # pylint: disable=broad-except
-                self.logger.exception(
-                    (
-                        "[%s(%s):%s] Exception caught while sending message to "
-                        "controller %s: %s"
-                    ),
-                    self.fn or "",
-                    self.host,
-                    self.port,
-                    type(self._handlers[message.namespace]).__name__,
-                    _message_to_string(message, data),
-                )
+                    if not handled:
+                        if data.get(REQUEST_ID) not in self._request_callbacks:
+                            self.logger.debug(
+                                "[%s(%s):%s] Message unhandled: %s",
+                                self.fn or "",
+                                self.host,
+                                self.port,
+                                _message_to_string(message, data),
+                            )
+                except Exception:  # pylint: disable=broad-except
+                    self.logger.exception(
+                        (
+                            "[%s(%s):%s] Exception caught while sending message to "
+                            "controller %s: %s"
+                        ),
+                        self.fn or "",
+                        self.host,
+                        self.port,
+                        type(handler).__name__,
+                        _message_to_string(message, data),
+                    )
 
         else:
             self.logger.debug(
@@ -740,11 +752,12 @@ class SocketClient(threading.Thread):
             except Exception:  # pylint: disable=broad-except
                 pass
 
-        for handler in self._handlers.values():
-            try:
-                handler.tear_down()
-            except Exception:  # pylint: disable=broad-except
-                pass
+        for namespace in self._handlers.values():
+            for handler in namespace:
+                try:
+                    handler.tear_down()
+                except Exception:  # pylint: disable=broad-except
+                    pass
 
         if self.socket is not None:
             try:
@@ -996,7 +1009,8 @@ class SocketClient(threading.Thread):
         """Handles a channel being disconnected."""
         for namespace in self.app_namespaces:
             if namespace in self._handlers:
-                self._handlers[namespace].channel_disconnected()
+                for handler in self._handlers[namespace]:
+                    handler.channel_disconnected()
 
         self.app_namespaces = []
         self.destination_id = None
